"""

    RTF document generator 

    Copyright 2006 Mikko Ohtamaa

"""

# Python imports
import StringIO

# Plone imports
from Products.Archetypes.Field import ObjectField
from Products.Archetypes.Field import DateTimeField

try:
    # Do we have DataGridField installed
    from Products.DataGridField.DataGridField import DataGridField
    USE_DATAGRID = True
except:
    USE_DATAGRID = False

# Local imports
from Products.RTFExport.PyRTF import *
from Products.RTFExport.PyRTF.Elements import Document
  

class RTFGenerator: 
    """ RTF generator
    
    Automatically Generates RTF document for a given Archetypes item
    
    To customize RTFGenerator for your item, you need to override two things:
        - exportableFields list (in __init__)
        - fillContsnts method            
    
    See examples for more details.
    
    Structure:
        - self.doc contains PyRTF document being generated
        - self.item contains Archetypes item being exported
        
    Walkthrough:
        - Initialize parameters
        - Generate PyRTF elements for the exported documents
        - Render PyRTF elements to .RTF file
        
    Optional support for DataGridField (add-on Archetypes field type) exists.
    
    """
    
    def __init__(self, item):
        """ Initialize one time RTF generator
        
        @param item Archetypes item
        @param exportableFields list of string names of fields which are exported
        """
        self.item = item
        self.doc = Document()
        ss = self.doc.StyleSheet
        
        # Initialize stylesheet
                
        # Add Bold style to stylesheet
        bold = TextStyle(TextPropertySet(ss.Fonts.Arial, 22, bold=True ), name="Bold" )                
        ss.TextStyles.append(bold)
        
        ps = ParagraphStyle('Bold',
                         bold.Copy(),
                         ParagraphPropertySet( space_before = 60,
                                               space_after  = 60 ) )
        ss.ParagraphStyles.append( ps )
        
        
    def generateRTF(self):
        """ Creates RTF document from item data
        
        @return document contents as a string
        """
        
        # fill in document
        self.generateContents()

        # String buffer IO object 
        buf = StringIO.StringIO()
        # PyRTF RTF renderer
        renderer = Renderer()
        renderer.Write(self.doc, buf)
        # Construct string
        return buf.getvalue()
    
    def getExportableFields(self):
        """ Return list of field names which are exported to RTF """
        raise "You must override RTFGenerator.getExportableFields method """
        
    def exportFields(self):
        """ Export Archetypes fields defined in getExportableFields() 
        
        """
                        
        for field in self.getExportableFields():
            self.exportField(field)

    def exportField(self, fieldName):
        """ Export one Archetypes field as RTF 
        
        Adds field data to current document body
        """
                
        field = self.item.schema[fieldName]
        ss = self.doc.StyleSheet
        section = self.section
        
        # Add field label 
        p = Paragraph( ss.ParagraphStyles.Heading2 )
        p.append(field.widget.label + ":")
        section.append( p )        
        
        if USE_DATAGRID and isinstance(field, DataGridField):
            self.exportDataGridField(field)
        elif isinstance(field, DateTimeField):
            self.exportDateTimeField(field)
        else:
            self.exportDefaultField(field)
            
        # Add empty line after a field
        p = Paragraph( ss.ParagraphStyles.Normal )
        p.append("")
        section.append( p )        
        
                    
    def exportDefaultField(self, field):
        """ Export Archetypes field as text value 
        
        This is fallback if no explict export method is defined: 
        Use str() to format field value
        """
        value = ObjectField.get(field, self.item) or ""
        
        ss = self.doc.StyleSheet
        
        p = Paragraph( ss.ParagraphStyles.Normal )
        p.append(str(value))
        self.section.append( p )  
        
    def exportDateTimeField(self, field):
        """ Export date/time as text value 
        
        Try use widget format parameter to format date/time
        """
        
        # code from archetypes/skins/widgets/calendar.pt
        d = ObjectField.get(field, self.item) or ""
        
        if hasattr(field.widget, "format") and d != "":
            format = field.widget.format
            result = d.strftime(format);
        else:            
            # TODO: add toPortalTime support instead of str(d)
            result = str(d)
                        
        ss = self.doc.StyleSheet
        
        p = Paragraph( ss.ParagraphStyles.Normal )
        p.append(result)
        self.section.append( p )          
        
    def exportDataGridField(self, field):
        """ Export Archetypes field as text value """     
        table = self.createDataGridTable(field)
        self.section.append(table)
        
    def generateContents(self):
        """ Generate contents for RTF
        
        Use PyRTF module to fill in RTF elements
        """
        ss = self.doc.StyleSheet

        # Section stored as an instance variable
        # so we don't need to pass it around
        self.section = Section()
        self.doc.Sections.append(self.section)
        
        self.fillContents()                 
        
        
    def fillContents(self):
        """ Override this function to customize your document """
        raise "You must override RTFGenerator.fillContents method """

        
    def createCell(self, table, friendlyName, attribute):
        """ Adds a field to table 
        
        Adds a table row 
        - Left cell contains friendlyName (bold)
        - Right cell contains attribute value
        
        @param table instance to add
        @param friendlyName Text for user
        @param attribute Archetypes attribute name
        
        @return None
        """
        ss = self.doc.StyleSheet
        c1 = Cell( Paragraph( ss.ParagraphStyles.Bold, friendlyName ) )
        c2 = Cell( Paragraph( ss.ParagraphStyles.Normal, str(attribute) ) )    
        table.AddRow( c1, c2)        
        
    def createDataGridTable(self, field):
        """ Export DataGridField as a table
        
        DataGridField is non-standard Archetypes field
        (available at plone.org/products)
        
        @param field DataGridField definition (for columns)
        @param instance DataGridField instance
        @return PyRTF table object
        """
        ss = self.doc.StyleSheet
    
        columns = field.getColumns(self.item)
        
        # Array of column widths
        columnWidths = []
        
        for c in columns:
            columnWidths.append(TabPS.DEFAULT_WIDTH*3)
        
        table = Table(*columnWidths)
        
        # Create table header
        for col in columns:
            c1 = Cell( Paragraph( ss.ParagraphStyles.Bold, col) )
            
        value = ObjectField.get(field, self.item) or []
        # Export table contents
        for row in value:
        
            # PyRTF cell objects for this table row
            cells = []
        
            for col in columns:
                # Read value from Archetypes field
                cellValue = row.get(col, '')
                cells.append(Cell(Paragraph(ss.ParagraphStyles.Normal, str(cellValue))))
                
            table.AddRow(*cells)
                
        return table
        
        
        
            
            
        
        